
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
DROLSKY
DROLSKY's
Rolsky
Rolsky's
AUTOHANDLERS
Adminstrator
ApacheModPerl
ApacheReload
Autohandlers
Bekman
CGI
ContactUs
DSO
DeWitt
DocumentRoot
DocumentRoots
FastCGI
FilesMatch
Follett
ForceFileDownload
GIF
Georgiou
HPUX
HUP
HandlingDirectoriesWithDhandlers
Khera
Kiriakos
Kirwan
Kumar
LFU
LogLevel
MSIE
MailingLists
Mallah
MasonAllowGlobals
MasonApacheStatusTitle
MasonArgsMethod
MasonAutoSendHeaders
MasonAutoflush
MasonAutohandlerName
MasonBufferPreallocateSize
MasonCodeCacheMaxSize
MasonCompClass
MasonCompRoot
MasonCompilerClass
MasonComponentErrorHandler
MasonDataCacheApi
MasonDataCacheDefaults
MasonDataDir
MasonDeclineDirs
MasonDefaultEscapeFlags
MasonDefineArgsHash
MasonDhandlerName
MasonDynamicCompRoot
MasonEnableAutoflush
MasonErrorFormat
MasonErrorMode
MasonEscapeFlags
MasonIgnoreWarningsExpr
MasonInPackage
MasonInterpClass
MasonLexerClass
MasonMaxRecurse
MasonNamedComponentSubs
MasonObjectFileExtension
MasonOutMethod
MasonPlugins
MasonPostamble
MasonPostprocessPerl
MasonPostprocessText
MasonPreamble
MasonPreloads
MasonPreprocess
MasonRequestClass
MasonResolverClass
MasonStaticSource
MasonStaticSourceTouchFile
MasonSubcompClass
MasonUseObjectFiles
MasonUseSourceLineNumbers
MasonUseStrict
MasonUseWarnings
NullCache
O'Reilly
PerlFreshRestart
PerlHandler
PerlModule
PerlSetVar
Preallocating
Preloading
RPMs
Rajesh
RedHat
ReloadAll
SUBCLASSABLE
SYNOPIS
Solaris
SpeedyCGI
Stas
Subcomponents
Subrequests
TIEHASH
USR
UserDir
Vivek
ala
apachectl
apachehandler
attr
autohandler
autohandlers
bgcolor
breakpoint'able
certian
checksum
conf
corrup
defined'ness
dhandler
dhandlers
dynamicImage
faq
fh
fido
filenaming
foobarbaz
gif
gifs
htaccess
html
interp
isNetscape
ized
izing
jpegs
lexed
libapreq
libexpat
mc
mcomp
mhtml
mpl
mtxt
nh
onwards
optimizations
overrideable
perlsub
postprocess
predeclaring
preload
preloaded
preloading
preloads
prepopulate
preprocess
profiler
rdist
reallocations
reparsed
reuseability
scomp
se
serializable
sql
srm
subcomponent
subcomponents
subcomps
subexec
subrequest
subrequests
taglibs
tgz
tmp
todo
un
undeclarable
unweakened
updateable
uring
use'd
xml
Jonathan
Swartz
swartz
Dave
autarch
Ken
Williams
ken
Ævar
Arnfjörð
Bjarmason
avarab
Alex
Vandiver
alex
John
jwilliams
Kevin
Falcone
falcone
Patrick
Kane
modus
Ricardo
Signes
rjbs
lib
HTML
Mason
Compiler
Devel
Params
Resolver
File
Plugin
Context
Utils
ToObject
FakeApache
Subclassing
Component
FileBased
Null
Escapes
Tests
Request
Subcomponent
Exceptions
Admin
Parser
Interp
Apache
Tools
Lexer
ApacheHandler
FAQ
Handler
CGIHandler
Cache
BaseCache
ComponentSource
MethodMaker
